import sinon from 'sinon'
import listenToClicksToTrack from '../../src/analytics/clickListener'

describe('#clickListener', () => {
  const FAKE_EVENT = 'myFakeEvent'
  const FAKE_ID = 'myFakeId'
  const FAKE_PROPS = '{&quot;section&quot;:&quot;candidate&quot;}'
  const FAKE_OPTIONS = '{&quot;protocols&quot;:{&quot;event_version&quot;:2}}'
  let buttonEl

  before(() => {
    window.rubik = window.rubik || {}
    window.rubik.analytics = window.rubik.analytics || {track: function() {}}
  })

  describe('when event, properties and options are set', () => {
    let useCaseSpy

    before(() => {
      useCaseSpy = sinon.spy()
      const fakeDomain = {
        get: () => ({
          execute: useCaseSpy
        })
      }

      const fixture = `
    <button
    id="${FAKE_ID}"
    data-track="${FAKE_EVENT}"
    data-track-options="${FAKE_OPTIONS}"
    data-track-properties="${FAKE_PROPS}"
    >
    Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack(fakeDomain)
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
    })

    it('should track the right event with its props and its options', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(useCaseSpy, {
        event: FAKE_EVENT,
        properties: {
          section: 'candidate'
        },
        options: {
          protocols: {event_version: 2}
        }
      })
    })
  })

  describe('when event and properties are set', () => {
    let useCaseSpy

    before(() => {
      useCaseSpy = sinon.spy()
      const fakeDomain = {
        get: () => ({
          execute: useCaseSpy
        })
      }

      const fixture = `
    <button
    id="${FAKE_ID}"
    data-track="${FAKE_EVENT}"
    data-track-properties="${FAKE_PROPS}"
    >
    Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack(fakeDomain)
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
    })

    it('should track the right event with its props', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(useCaseSpy, {
        event: FAKE_EVENT,
        properties: {
          section: 'candidate'
        },
        options: {}
      })
    })
  })
  describe('when event is set without properties', () => {
    let useCaseSpy
    before(() => {
      useCaseSpy = sinon.spy()
      const fakeDomain = {
        get: () => ({
          execute: useCaseSpy
        })
      }

      const fixture = `
    <button
      id="${FAKE_ID}"
      data-track="${FAKE_EVENT}"
      >
        Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack(fakeDomain)
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
    })

    it('should track the right event with no props', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(useCaseSpy, {
        event: FAKE_EVENT,
        properties: {},
        options: {}
      })
    })
  })

  describe('when no event is set ', () => {
    let useCaseSpy
    before(() => {
      useCaseSpy = sinon.spy()
      const fakeDomain = {
        get: () => ({
          execute: useCaseSpy
        })
      }

      const fixture = `
    <button
      id="${FAKE_ID}"
      >
        Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack(fakeDomain)
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
    })

    it('should not call the track function', () => {
      buttonEl.click()
      sinon.assert.callCount(useCaseSpy, 0)
    })
  })

  describe('when the properties does not contain quot', () => {
    let useCaseSpy
    before(() => {
      useCaseSpy = sinon.spy()
      const fakeDomain = {
        get: () => ({
          execute: useCaseSpy
        })
      }
      const fixture = `
    <button
    id="${FAKE_ID}"
    data-track="${FAKE_EVENT}"
    data-track-properties='{"key1": "value1", "key2": 2}'
    data-track-options='{"optionKey": "fakeValue"}'
    >
    Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack(fakeDomain)
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
    })

    it('should track the right event with its props', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(useCaseSpy, {
        event: FAKE_EVENT,
        properties: {
          key1: 'value1',
          key2: 2
        },
        options: {
          optionKey: 'fakeValue'
        }
      })
    })
  })
})
